# Copyright (c) Microsoft Corporation.  All rights reserved.

from ._utils import record_new_telemetry_for_top_level
from ._derive_value_result import DeriveValueResult


@record_new_telemetry_for_top_level
def derive_value(input_table=None, options=None):
    """
    Generates code for computing values of a new column based on the given options.

    :param input_table: The full set of input values that learning operations should consider.
    :type input_table: Must be a value that can be passed to the pandas DataFrame constructor
    (`list`, `dict`, `DataFrame`, etc).
    :param options: Options to use when learning code to compute new column from old.
    :type options: Must be a value that can be passed to the pandas DataFrame constructor
    :type: :class:'prose.codeaccelerator.DeriveValueOptions`.
    :return: The result of learning.
    :rtype: :class:'prose.codeaccelerator.DeriveValueResult`.
    """
    return lambda logger: DeriveValueResult(logger, input_table, options)


@record_new_telemetry_for_top_level
def _derive_value(mime_data):
    """Internal version of the derive_value entry point function which takes mime
    bundle data object, runs the learn call and returns an updated mime bundle
    data object.
    """

    def logged_derive_value(logger):
        result = DeriveValueResult._from_serializable(logger, mime_data)
        return result.to_serializable()

    return logged_derive_value
