# Copyright (c) Microsoft Corporation.  All rights reserved.

from ._utils import _serialize_dataframe, _deserialize_dataframe
import pandas as pd


class DeriveValueOptions:
    """Options specific to creating a new column using data from existing ones."""

    def __init__(
        self,
        examples=None,
        input_column_names=None,
        new_column_name="Examples",  # temporary change for Azure Notebooks demo, default column should be 'New Column'
        bound_names=[],
        input_table_variable_name="df",
        index_based_examples=[],
    ):
        """Initializes options for creating a function to derive a value.

        :param examples: Input-output examples that specify the behavior of the intended program.
        :type: dict: Tuple[str | int | float | datetime ] -> str
        :param input_column_names: The subset of column names (from the df) to use to generate the new column.
        :type: list of str
        :param new_column_name: The name to use as the column name for the new column.
        :type: str
        :param bound_names: The names that should not be used in the generated code (because, for e.g, they are live in the users context.)
        :type: list of str
        :param input_table_variable_name: The name to use to access the input table in generated code snippet.
        :type: str
        :param index_based_examples: Same as examples, but where the input values come from the current values in the dataframe at a specified row # rather than being specified explicitly.
        :type: list of dict
        """
        self.examples = examples
        self.suggested_examples = None
        self.input_column_names = input_column_names
        self.new_column_name = new_column_name
        self._bound_names = bound_names or []
        self._input_table_variable_name = input_table_variable_name
        self._index_based_examples = index_based_examples

    def __repr__(self):
        return """DeriveValueOptions
===============
examples = {}
input_column_names = {}
suggested_examples = {}
new_column_name = {}
""".format(
            self.examples,
            self.input_column_names,
            self.suggested_examples,
            self.new_column_name,
        )
