# Copyright (c) Microsoft Corporation.  All rights reserved.

from prose.codeaccelerator import _internals
from ._read_options import ReadOptions
from .._buffered_reader import ReadTextBufferedReader
from .._config import config


class ReadHtmlOptions(ReadOptions):
    """Options specific to reading HTML files."""

    @property
    def _internal(self):
        return self._read_html_options

    def __init__(self, target=None, detect_data_types=True):
        """Initializes options for reading HTML file.

        :param target: Target platform for generated code.
        """
        self._read_html_options = _internals.ReadHtmlOptions()
        self.target = target if target else ReadHtmlOptions._default_target()
        self.detect_data_types = detect_data_types

    @classmethod
    def _default_target(cls):
        """Default target to use for this option."""
        return config.get("default_target", "pandas")

    def _allowed_targets(self):
        """Specify a list of the allowed targets for this builder type."""
        return self._read_html_options.allowed_targets

    @property
    def target(self):
        """Target runtime library for the generated code.

        Supported values are :code:`"pandas"` and :code:`"pyspark"`.

        :type: str
        """
        return self._read_html_options.target

    @target.setter
    def target(self, value):
        value = super()._validate_target(value)
        self._read_html_options.target = value

    @property
    def encoding(self):
        """The detected encoding of the sample file that will be used for subsequent reads.

        :type: str
        """
        return self._read_html_options.encoding

    def to_serializable(self):
        return {"target": self.target, "encoding": self._read_html_options.encoding}

    @classmethod
    def from_serializable(cls, serializable):
        return cls(serializable.get("target"), serializable.get("encoding"))

    def __repr__(self):
        return """\
ReadHtmlOptions
===============
target = {}
encoding = {}""".format(
            self.target, self.encoding
        )
