# Copyright (c) Microsoft Corporation.  All rights reserved.
from abc import ABC, abstractmethod


class ReadOptions(ABC):
    @property
    @abstractmethod
    def _internal(self):
        """Get the C# internal object that is backing this object."""

    @abstractmethod
    def _allowed_targets(self):
        """Specify a list of the allowed targets for this builder type."""

    def _validate_target(self, value):
        lower_value = value.lower()
        if lower_value not in self._allowed_targets():
            raise ValueError(
                "Invalid value '{}' for 'target'. Expected a value from {}.".format(
                    value, self._allowed_targets()
                )
            )
        return lower_value

    @property
    def detect_data_types(self):
        """Whether or not to synthesize code to detect data types as part of reading the file.
        :return: True, if data type detection is enabled, False otherwise.
        :rtype: bool
        """
        return self._internal.detect_data_types

    @detect_data_types.setter
    def detect_data_types(self, value):
        self._internal.detect_data_types = value
