# Copyright (c) Microsoft Corporation.  All rights reserved.

from prose.codeaccelerator import _internals
from .._config import config
from ._read_options import ReadOptions


class ReadParquetOptions(ReadOptions):
    """Options specific to reading parquet files."""

    @property
    def _internal(self):
        return self._read_parquet_options

    def __init__(self, target=None, columns=None, detect_data_types=True):
        """Initializes options for reading parquet file.

        :param target: Target platform for generated code.
        :param columns: Names of columns that will be read from the file.
        :param detect_data_types: Whether or not to synthesize code to detect data types as part of reading the file.
        """
        self._read_parquet_options = _internals.ReadParquetOptions()
        self.target = target if target else ReadParquetOptions._default_target()
        self.columns = columns
        self.detect_data_types = detect_data_types

    @classmethod
    def _default_target(cls):
        """Default target to use for this option."""
        return config.get("default_target", "pandas")

    def _allowed_targets(self):
        """Specify a list of the allowed targets for this builder type."""
        return self._read_parquet_options.allowed_targets

    @property
    def target(self):
        """Target runtime library for the generated code.

        Supported values are :code:`"pandas"` and :code:`"pyspark"`. Default is :code:`"pandas"`.

        :type: str
        """
        return self._read_parquet_options.target

    @target.setter
    def target(self, value):
        value = super()._validate_target(value)
        self._read_parquet_options.target = value

    @property
    def columns(self):
        """ List of column names to be read from file.
        :return: Names of columns that will be read from the file.
        :rtype: list
        """
        return self._read_parquet_options.columns

    @columns.setter
    def columns(self, value):
        self._read_parquet_options.columns = value

    def __repr__(self):
        return """ReadParquetOptions
===============
columns = {}
detect_data_types = {}
target = {}""".format(
            self.columns, self.detect_data_types, self.target
        )
