# Copyright (c) Microsoft Corporation.  All rights reserved.

from io import BufferedReader


class ReadTextBufferedReader(BufferedReader):
    def __init__(self, input_file, create_stream):
        """
        Creates a new instance of buffered reader to read the input provided for file ingestion scenarios.
        :param input_file:
        :param create_stream:
        """
        if create_stream:
            input_file = open(input_file, "rb")

        super().__init__(input_file)
        self._close_stream = create_stream
        if create_stream:
            self._original_position = 0
        else:
            self._original_position = input_file.tell()
        self._initialized = True

    def reset_position(self):
        """
        Resets the stream to original position that was present when it was given during
        this instance construction.
        :return:
        """
        super().seek(self._original_position)

    def __del__(self):
        if hasattr(self, "_initialized") and not self.closed:
            # If we created the stream, then close it
            if self._close_stream:
                super().__del__()
            # else, just detach
            else:
                self.detach()
