# Copyright (c) Microsoft Corporation.  All rights reserved.

from . import _utils


class BaseFunction(object):
    def __init__(self, function_name, program_string):
        assert function_name is not None
        assert program_string is not None

        self.function_name = function_name
        self.function_string = program_string
        self.context = {k: v for k, v in globals().items()}
        self.initialized = False

    def _get_function(self):
        self._initialize()
        return self.context.get(self.function_name, None)

    @_utils.executes_generated_code
    def _initialize(self):
        if not self.initialized:
            exec(self.function_string, self.context)
            self.initialized = True

    @_utils.executes_generated_code
    def _generic_call(self, *args):
        function = self._get_function()
        return function(*args) if function is not None else None


class OneArgFunction(BaseFunction):
    def __init__(self, function_name, program_string):
        super().__init__(function_name, program_string)

    def __call__(self, arg):
        return self._generic_call(arg)

    def __repr__(self):
        return self.function_string


class TwoArgFunction(BaseFunction):
    def __init__(self, function_name, program_string):
        super().__init__(function_name, program_string)

    def __call__(self, arg1, arg2):
        return self._generic_call(arg1, arg2)

    def __repr__(self):
        return self.function_string


class VarArgFunction(BaseFunction):
    def __init__(self, function_name, program_string):
        super().__init__(function_name, program_string)

    def __call__(self, *args):
        return self._generic_call(*args)

    def __repr__(self):
        return self.function_string
