# Copyright (c) Microsoft Corporation.  All rights reserved.
# This file is auto-generated. Do not make manual changes.

from .protobuf_objects import ProseMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2
from .protobuf_objects import ProseDispatchApisMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2
from .protobuf_objects import ProseFetchApisMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2
from .background_process import BackgroundProcess, _ProcessState
from .cache_manager import CacheManager
from .protobuf_objects.background_process_terminate import process_terminator
from .null_value_converter_utils import NullValueConverterUtils


class RegexTokenSpecApi(object):
    def __init__(self, background_process, name, regexPattern, score, prepopulated_referenceId=None):
        """
        Construct a regex token spec, given the name, pattern, and score.
        :param name: The name of the custom regex token.
        :param regexPattern: The regex pattern for the custom regex token.
        :param score: The score of the custom regex token.
        :return:
        """
        assert background_process is not None
        self._background_process = background_process
        self._cache_manager = CacheManager(id(background_process))

        # if object was already created in C#, just initialize the referenceId.
        if prepopulated_referenceId is not None:
            self.referenceId = prepopulated_referenceId
            self._cache_manager.add_object(self.referenceId, self)
            return
        # else if object is constructed in Python, then create an equivalent in C#.

        # Create a request object for dispatcher
        message_request = ProseMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2.PythonToCSharp()
        request = message_request.requestForDispatcher


        # Extract 'RegexTokenSpec' object from request and set parameters
        regex_token_spec_request = request.iRegexTokenSpec
        NullValueConverterUtils.convert_to_pb_nullable_string(regex_token_spec_request.i_In_RegexTokenSpec___init__.name, name)
        NullValueConverterUtils.convert_to_pb_nullable_string(regex_token_spec_request.i_In_RegexTokenSpec___init__.regexPattern, regexPattern)
        regex_token_spec_request.i_In_RegexTokenSpec___init__.score = score
        message_response = ProseMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2.CSharpToPython()

        # Communicate with Coreclr
        received_bytes = self._background_process.communicate(message_request)

        # If we didn't receive any bytes, raise Error
        if received_bytes is None:
            raise RuntimeError('Internal error has occurred. Please retry the entire scenario.')

        message_response.ParseFromString(received_bytes)
        response = message_response.responseFromDispatcher

        # Check if any exception was thrown by RegexTokenSpec..ctor
        if response.WhichOneof('ScenarioOutput') == 'Exception':
            raise RuntimeError(response.Exception.message)
        # Ensure we have a valid referenceId
        if response.referenceId is 0:
            raise ValueError('Failed to create RegexTokenSpec object.')
        self.referenceId = response.referenceId
        # Cache the object
        self._cache_manager.add_object(self.referenceId, self)

    def _get_name(self):

        # Create a request object for dispatcher
        message_request = ProseMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2.PythonToCSharp()
        request = message_request.requestForDispatcher

        request.referenceId = self.referenceId

        # Extract 'RegexTokenSpec' object from request and set parameters
        regex_token_spec_request = request.iRegexTokenSpec
        regex_token_spec_request.i_In_RegexTokenSpec__get_Name.SetInParent()
        message_response = ProseMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2.CSharpToPython()

        # Communicate with Coreclr
        received_bytes = self._background_process.communicate(message_request)

        # If we didn't receive any bytes, raise Error
        if received_bytes is None:
            raise RuntimeError('Internal error has occurred. Please retry the entire scenario.')

        message_response.ParseFromString(received_bytes)
        response = message_response.responseFromDispatcher

        # Check if any exception was thrown by RegexTokenSpec.Name
        if response.WhichOneof('ScenarioOutput') == 'Exception':
            raise RuntimeError(response.Exception.message)

        # Process the result
        regex_token_spec_response = response.iRegexTokenSpec.i_Out_RegexTokenSpec__get_Name
        to_return = NullValueConverterUtils.convert_from_pb_nullable_string(regex_token_spec_response.result)
        return to_return

    def _get_regex_pattern(self):

        # Create a request object for dispatcher
        message_request = ProseMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2.PythonToCSharp()
        request = message_request.requestForDispatcher

        request.referenceId = self.referenceId

        # Extract 'RegexTokenSpec' object from request and set parameters
        regex_token_spec_request = request.iRegexTokenSpec
        regex_token_spec_request.i_In_RegexTokenSpec__get_RegexPattern.SetInParent()
        message_response = ProseMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2.CSharpToPython()

        # Communicate with Coreclr
        received_bytes = self._background_process.communicate(message_request)

        # If we didn't receive any bytes, raise Error
        if received_bytes is None:
            raise RuntimeError('Internal error has occurred. Please retry the entire scenario.')

        message_response.ParseFromString(received_bytes)
        response = message_response.responseFromDispatcher

        # Check if any exception was thrown by RegexTokenSpec.RegexPattern
        if response.WhichOneof('ScenarioOutput') == 'Exception':
            raise RuntimeError(response.Exception.message)

        # Process the result
        regex_token_spec_response = response.iRegexTokenSpec.i_Out_RegexTokenSpec__get_RegexPattern
        to_return = NullValueConverterUtils.convert_from_pb_nullable_string(regex_token_spec_response.result)
        return to_return

    def _get_score(self):

        # Create a request object for dispatcher
        message_request = ProseMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2.PythonToCSharp()
        request = message_request.requestForDispatcher

        request.referenceId = self.referenceId

        # Extract 'RegexTokenSpec' object from request and set parameters
        regex_token_spec_request = request.iRegexTokenSpec
        regex_token_spec_request.i_In_RegexTokenSpec__get_Score.SetInParent()
        message_response = ProseMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2.CSharpToPython()

        # Communicate with Coreclr
        received_bytes = self._background_process.communicate(message_request)

        # If we didn't receive any bytes, raise Error
        if received_bytes is None:
            raise RuntimeError('Internal error has occurred. Please retry the entire scenario.')

        message_response.ParseFromString(received_bytes)
        response = message_response.responseFromDispatcher

        # Check if any exception was thrown by RegexTokenSpec.Score
        if response.WhichOneof('ScenarioOutput') == 'Exception':
            raise RuntimeError(response.Exception.message)

        # Process the result
        regex_token_spec_response = response.iRegexTokenSpec.i_Out_RegexTokenSpec__get_Score
        to_return = regex_token_spec_response.result
        return to_return

    name = property(_get_name, None, None, 'accessor for Name')
    regex_pattern = property(_get_regex_pattern, None, None, 'accessor for RegexPattern')
    score = property(_get_score, None, None, 'accessor for Score')
    def __del__(self):

        # If we haven't initialized successfully, just return
        if not hasattr(self, 'referenceId'):
            return
        # Create a request object for dispatcher
        message_request = ProseMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2.PythonToCSharp()
        request = message_request.requestForDispatcher

        request.referenceId = self.referenceId
        request.DeleteFromCache.SetInParent()
        message_response = ProseMicrosoftProgramSynthesisPythonCodeAcceleratorInternals_pb2.CSharpToPython()
        # Communicate with Coreclr
        received_bytes = self._background_process.communicate(message_request)

        # If we didn't receive any bytes, just return
        if received_bytes is None:
            return

        message_response.ParseFromString(received_bytes)
        response = message_response.responseFromDispatcher

        # Check if any exception was thrown by RegexTokenSpec.__del__
        if response.WhichOneof('ScenarioOutput') == 'Exception':
            raise RuntimeError(response.Exception.message)

    __slots__ = ['_background_process', '_cache_manager', 'referenceId', '__weakref__']


class RegexTokenSpec(RegexTokenSpecApi):
    def __init__(self, name, regexPattern, score, prepopulated_referenceId=None):
        """
        Construct a regex token spec, given the name, pattern, and score.
        :param name: The name of the custom regex token.
        :param regexPattern: The regex pattern for the custom regex token.
        :param score: The score of the custom regex token.
        :return:
        """
        background_process = BackgroundProcess(r'Microsoft.ProgramSynthesis.Python.CodeAccelerator.Internals.dll', '04/18/2020 05:15:37', process_terminator)
        assert background_process._process_state == _ProcessState.RUNNING
        super().__init__(background_process, name, regexPattern, score, prepopulated_referenceId)

    def __del__(self):
        super().__del__()

    __slots__ = []

    @classmethod
    def prepopulated_object_builder(cls, prepopulated_referenceId):
        """
        Method to construct RegexTokenSpec object if equivalent C# object is already created
        """

        return RegexTokenSpec(None, None, None, prepopulated_referenceId)
