# Copyright (c) Microsoft Corporation.  All rights reserved.
import pathlib

from ._api._read_text_options import ReadTextOptions
from ._buffered_reader import ReadTextBufferedReader
from .exceptions import LearnError
from ._result import ResultImpl
from . import _internals, _code
from ._detect_types_result import DetectTypesResultImpl


class ReadTextResultImpl(ResultImpl):
    @property
    def _internal(self):
        return self._result

    def __init__(self, logger, filename, options):
        self._filename = 'r"{}"'.format(str(pathlib.Path(filename)))
        self._options = ReadTextOptions() if options is None else options
        self._code_snippet = None
        self._code_string = None
        self._reader = ReadTextBufferedReader(filename, create_stream=True)
        self._result = _internals.ReadTextResult(logger, filename, self._reader)
        self.learn()

    @property
    def code(self):
        return self._code_string

    @property
    def options(self):
        return self._options

    @property
    def data(self):
        if self._code_snippet is None:
            return None

        return self._code_snippet.evaluate()

    def learn(self):
        self._result.learn(self.options._read_text_options)
        if self._result.error is not None:
            raise LearnError(self._result.error)
        self._code_string = self._result.code
        self._code_snippet = _code.CodeSnippet(self._code_string)

        if self._options.detect_data_types:
            data = self.data
            self._code_snippet = DetectTypesResultImpl.detect_data_types(
                self.options.target,
                self._code_string,
                data,
                self._internal.python_logger,
                "df",
            )
            self._code_string = repr(self._code_snippet)

    def __repr__(self):
        return """ReadTextResult
===============
filename = {}
options = {}""".format(
            self._filename, self.options
        )
