# Copyright (c) Microsoft Corporation.  All rights reserved.

from . import _config, _version
from .exceptions import ProseException

# constants
PREVIEW_DATA_LIMIT = 5


def shortened_repr(l, length=5, ellipsis="..."):
    if len(l) <= length:
        return repr(l)
    element_reprs = list(map(repr, l[:5]))
    return "[ {0} ]".format(", ".join(element_reprs + ["..."]))


def shortened_dict_repr(d, length=5, ellipsis="..."):
    if not d:
        return ""
    return "\n".join(
        [
            "{}: {}".format(repr(k), shortened_repr(v, length, ellipsis))
            for k, v in sorted(d.items())
        ]
    )


def executes_generated_code(func):
    """Decorator that wraps functions to handle any potential exceptions that might be thrown by prose generated code.

    We wrap all execution of prose generated code to handle any difference between the target environment and the design environment.

    :param func: The function to wrap.
    """

    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except ImportError as e:
            if e.name == "pyspark":
                raise ProseException(
                    "Target of the code builder is set to pyspark.  To successfully run the generated code on this machine, you must install pyspark.  To generate code targeting pandas, set the target to pandas."
                )
            else:
                raise

    return wrapper


def _telemetry_settings():
    return (
        # The Python SDK version
        _version.__version__,
        # The host name that we will record in the telemetry logging
        _config.config["host_name"],
        # Whether to disable telemetry
        _config.config["telemetry_optout"],
        # Whether the telemetry logging is in testing mode
        _config.config["telemetry_testing_mode"],
    )
